package com.school.project.service.teacher;

import com.school.project.dto.teacher.AddAssessmentDTO;
import com.school.project.dto.teacher.UploadMarksDTO;
import com.school.project.mapper.AssessmentToListOfAssessment;
import com.school.project.model.Course;
import com.school.project.model.Student;
import com.school.project.model.marks.Assessment;
import com.school.project.model.marks.Section;
import com.school.project.model.marks.StudentMarks;
import com.school.project.repository.CourseRepository;
import com.school.project.repository.StudentRepository;
import com.school.project.repository.marks.AssessmentRepository;
import com.school.project.repository.marks.AssessmentTypeRepository;
import com.school.project.repository.marks.SectionRepository;
import com.school.project.repository.marks.StudentMarksRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;

@Service
public class UploadMarksServiceImpl implements UploadMarksService {

    @Autowired
    CourseRepository courseRepository;

    @Autowired
    StudentRepository studentRepository;

    @Autowired
    AssessmentTypeRepository assessmentTypeRepository;

    @Autowired
    AssessmentRepository assessmentRepository;

    @Autowired
    SectionRepository sectionRepository;

    @Autowired
    StudentMarksRepository studentMarksRepository;

    @Autowired
    AssessmentToListOfAssessment assessmentMapper;

    @Override
    @Transactional
    public void setAssessment(AddAssessmentDTO assessmentDTO) {

        Assessment assessment = new Assessment();

        Course course = courseRepository.findByCourseCode(assessmentDTO.getCourseCode());

        assessment.setAssessmentType(assessmentTypeRepository.findByAssessmentTypeName(assessmentDTO.getAssessment()));

        assessment.setCourse(course);

        assessmentRepository.save(assessment);
    }

    @Override
    @Transactional
    public void setObtainedMarks(UploadMarksDTO uploadMarksDTO) {

        Assessment assessment = assessmentRepository.findByAssessmentId(uploadMarksDTO.getAssessmentId());

        uploadMarksDTO.getSectionsList().forEach(sectionInList -> {

                    Section newSection = new Section();

                    newSection.setTotalMarks(sectionInList.getMarks());
                    newSection.setTotalWeightage(sectionInList.getWeightage());
                    newSection.setAssessment(assessment);

                    Section section = sectionRepository.save(newSection);

                    sectionInList.getObtainedMarksDTOS().forEach(obtainedMarksInList -> {

                        Student student = studentRepository.findByRollNumber(obtainedMarksInList.getStudentRollNumber());

                        StudentMarks studentMarks = new StudentMarks();

                        studentMarks.setObtainedMarks(obtainedMarksInList.getObtainedMarks());
                        studentMarks.setStudent(student);
                        studentMarks.setSection(section);

                        studentMarksRepository.save(studentMarks);

                    });

                }
        );

    }
}
