package com.school.project.service.userRegistration.student;

import com.school.project.dto.userRegistration.StudentRegistrationDTO;
import com.school.project.dto.userRegistration.UserRegistrationDTO;
import com.school.project.model.userRegistration.Role;
import com.school.project.model.Student;
import com.school.project.model.userRegistration.User;
import com.school.project.repository.userRegistration.RoleRepository;
import com.school.project.repository.StudentRepository;
import com.school.project.repository.userRegistration.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;

@Service
public class StudentRegistrationServiceImpl implements StudentRegistrationService {

    @Autowired
    private StudentRepository studentRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private RoleRepository roleRepository;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Override
    @Transactional
    public UserRegistrationDTO registerStudent(StudentRegistrationDTO studentRegistrationDTO) {

        User newUser = new User();

        newUser.setEmail(studentRegistrationDTO.getEmail());
        newUser.setRole(roleRepository.findByRole(Role.ROLES.STUDENT.name()));
        newUser.setPassword(passwordEncoder.encode(studentRegistrationDTO.getPassword()));

        newUser = userRepository.save(newUser);

        Student newStudent = new Student();

        newStudent.setStudentName(studentRegistrationDTO.getName());
        newStudent.setAge(studentRegistrationDTO.getAge());
        newStudent.setStudentAppUser(newUser);
        newStudent.setRollNumber(studentRegistrationDTO.getRollNumber());

        studentRepository.save(newStudent);

        return new UserRegistrationDTO(newUser.getUserId(), newUser.getEmail());

    }
}
