package com.school.project.service.userRegistration.teacher;

import com.school.project.dto.userRegistration.TeacherRegistrationDTO;
import com.school.project.dto.userRegistration.UserRegistrationDTO;
import com.school.project.model.userRegistration.Role;
import com.school.project.model.Teacher;
import com.school.project.model.userRegistration.User;
import com.school.project.repository.userRegistration.RoleRepository;
import com.school.project.repository.TeacherRepository;
import com.school.project.repository.userRegistration.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;

@Service
public class TeacherRegistrationServiceImpl implements TeacherRegistrationService {

    @Autowired
    private TeacherRepository teacherRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private RoleRepository roleRepository;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Override
    @Transactional
    public UserRegistrationDTO registerTeacher(TeacherRegistrationDTO teacherRegistrationDTO) {

        User newUser = new User();

        newUser.setEmail(teacherRegistrationDTO.getEmail());
        newUser.setRole(roleRepository.findByRole(Role.ROLES.TEACHER.name()));
        newUser.setPassword(passwordEncoder.encode(teacherRegistrationDTO.getPassword()));

        newUser = userRepository.save(newUser);

        Teacher newTeacher = new Teacher();

        newTeacher.setName(teacherRegistrationDTO.getName());
        newTeacher.setDegree(teacherRegistrationDTO.getDegree());
        newTeacher.setTeacherNumber(teacherRegistrationDTO.getTeacherNumber());
        newTeacher.setTeacherAppUser(newUser);

        teacherRepository.save(newTeacher);

        return new UserRegistrationDTO(newTeacher.getTeacherId(), newUser.getEmail());

    }
}
